"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.HibernatingSwitch = void 0;
const limited_cache_1 = require("limited-cache");
const react_1 = __importStar(require("react"));
const react_is_1 = require("react-is");
const react_reverse_portal_1 = require("react-reverse-portal");
const react_router_1 = require("react-router");
const HibernatingRoute_1 = require("./HibernatingRoute");
const renderRoute_1 = require("./renderRoute");
const HibernatingSwitch = (_a) => {
    var { children, maxCacheSize, maxCacheTime, WrapperComponent } = _a, allOtherProps = __rest(_a, ["children", "maxCacheSize", "maxCacheTime", "WrapperComponent"]);
    const portalRecordCache = (0, react_1.useMemo)(() => (0, limited_cache_1.LimitedCache)({
        maxCacheSize,
        maxCacheTime,
    }), [maxCacheSize, maxCacheTime]);
    const currentPathKeyRef = (0, react_1.useRef)();
    const currentPathKey = currentPathKeyRef.current;
    const [currentPortalRecord, setCurrentPortalRecord] = (0, react_1.useState)(null);
    if (process.env.NODE_ENV !== 'production') {
        // eslint-disable-next-line react-hooks/rules-of-hooks
        const InitialWrapperComponentRef = (0, react_1.useRef)(WrapperComponent);
        if (WrapperComponent !== InitialWrapperComponentRef.current) {
            console.warn('The WrapperComponent component changed between renders: this will cause a remount');
            InitialWrapperComponentRef.current = WrapperComponent;
        }
    }
    const activatePortalForComponent = (routerProps, routeProps) => {
        // We don't really care about the *path* that was matched. Instead, we care about the *url* that activated us
        const pathKey = routerProps.match.url;
        if (pathKey === currentPathKey) {
            // Nothing has really changed: just update any props
            currentPortalRecord.routerProps = routerProps;
            currentPortalRecord.routeProps = routeProps;
        }
        else {
            // New route! Stash the old and move to the new
            // But first, pull any previous portal record for the 'new' screen (in case it's in the cache) --
            // otherwise it might get removed when we add the old portal record, if the cache size is small.
            const previousPortalRecord = portalRecordCache.get(pathKey);
            if (currentPathKey) {
                portalRecordCache.set(currentPathKey, currentPortalRecord);
            }
            let newPortalRecord;
            if (previousPortalRecord) {
                // Reactivate the prior subtree
                previousPortalRecord.routerProps = routerProps;
                previousPortalRecord.routeProps = routeProps;
                newPortalRecord = previousPortalRecord;
            }
            else {
                // Make a new portal for the new subtree
                newPortalRecord = {
                    portalNode: (0, react_reverse_portal_1.createHtmlPortalNode)(),
                    routerProps,
                    routeProps,
                };
            }
            currentPathKeyRef.current = pathKey;
            setCurrentPortalRecord(newPortalRecord);
        }
    };
    const deactivatePortal = () => {
        if (currentPathKey && currentPortalRecord) {
            portalRecordCache.set(currentPathKey, currentPortalRecord);
            // We need to unmount the old OutPortal *immediately* so that it can swap in its original dom node,
            // or else React hits an error trying to unmount in the incorrect node
            currentPortalRecord.portalNode.unmount();
            currentPathKeyRef.current = '';
            setCurrentPortalRecord(null);
        }
    };
    const childrenWithHibernation = react_1.default.Children.map(children, (child) => {
        if ((0, react_is_1.isElement)(child)) {
            const { type, props: routeProps } = child, 
            // eslint-disable-next-line @typescript-eslint/no-unused-vars
            _a = child.props, 
            // eslint-disable-next-line @typescript-eslint/no-unused-vars
            { children, component, isHibernatingRoute, render } = _a, allOtherRouteProps = __rest(_a, ["children", "component", "isHibernatingRoute", "render"]);
            if (type === HibernatingRoute_1.HibernatingRoute || isHibernatingRoute) {
                // Replace it: it will activate the remote node when the route matches
                return (react_1.default.createElement(react_router_1.Route, Object.assign({}, allOtherRouteProps, { render: (routerProps) => {
                        activatePortalForComponent(routerProps, routeProps);
                        return null;
                    } })));
            }
            if (type !== react_router_1.Redirect && isHibernatingRoute !== false) {
                // Every child should either be a Hibernating Route, vanilla Route, or Redirect -- and if it's neither a
                // vanilla route nor a Hibernating Route, it's probably a custom wrapper around Route.
                // If it's any non-Hibernating route Route then we need to deactivate any portal which might be active from
                // a Hibernating Route. But no matter what it might be, we want to deactivate before rendering, to be safe.
                return (react_1.default.createElement(react_router_1.Route, Object.assign({}, allOtherRouteProps, { render: (routerProps) => {
                        deactivatePortal();
                        return (0, renderRoute_1.renderRoute)(routerProps, routeProps);
                    } })));
            }
        }
        // If it's a Redirect, or anything we can't recognize, let it pass through as-is
        return child;
    });
    const portalRecordCacheFull = portalRecordCache.getAll();
    const allPortalKeys = Object.keys(portalRecordCacheFull);
    if (currentPathKey && !allPortalKeys.includes(currentPathKey)) {
        allPortalKeys.push(currentPathKey);
    }
    return (react_1.default.createElement(react_1.default.Fragment, null,
        react_1.default.createElement(react_router_1.Switch, Object.assign({}, allOtherProps), childrenWithHibernation),
        react_1.default.createElement(react_1.default.Fragment, null,
            allPortalKeys.map((pathKey) => {
                const portalRecord = pathKey === currentPathKey ? currentPortalRecord : portalRecordCacheFull[pathKey];
                if (!portalRecord) {
                    if (process.env.NODE_ENV !== 'production') {
                        console.warn(`portalRecord is missing for pathKey "${pathKey}"`);
                    }
                    return null;
                }
                const { portalNode, routerProps, routeProps } = portalRecord;
                const routeContent = (0, renderRoute_1.renderRoute)(routerProps, routeProps);
                const wrappedRouteContent = WrapperComponent ? (react_1.default.createElement(WrapperComponent, { shouldUpdate: pathKey === currentPathKey }, routeContent)) : (routeContent);
                return (react_1.default.createElement(react_reverse_portal_1.InPortal, { key: pathKey, node: portalNode }, wrappedRouteContent));
            }),
            !!currentPortalRecord && react_1.default.createElement(react_reverse_portal_1.OutPortal, { node: currentPortalRecord.portalNode }))));
};
exports.HibernatingSwitch = HibernatingSwitch;
HibernatingSwitch.defaultProps = {
    maxCacheSize: 5,
    maxCacheTime: 5 * 60 * 1000,
    WrapperComponent: null,
};
//# sourceMappingURL=HibernatingSwitch.js.map